@@ -27,6 +27,8 @@ pub static SUCCESS: &str = "SUCCESS";
 pub enum Error {
     #[error("Client is not authorized. No bearer token available")]
     NoToken,
+    #[error("Invalid customer ip. IP 0.0.0.0 is not acceptable")]
+    CustomerIp,
     #[error("{0}")]
     Io(#[from] std::io::Error),
     #[error("Total value is not sum of products price")]
@@ -196,19 +198,86 @@ pub enum RefundStatus {
     Completed,
 }
 
+#[derive(Serialize, Deserialize, Debug, Default)]
+#[serde(rename_all = "camelCase")]
+pub struct Delivery {
+    #[serde(skip_serializing_if = "Option::is_none")]
+    /// Street name
+    street: Option<String>,
+
+    #[serde(skip_serializing_if = "Option::is_none")]
+    /// Postal box number
+    postal_box: Option<String>,
+
+    #[serde(skip_serializing_if = "Option::is_none")]
+    /// Postal code
+    postal_code: Option<String>,
+
+    #[serde(skip_serializing_if = "Option::is_none")]
+    /// City
+    city: Option<String>,
+
+    #[serde(skip_serializing_if = "Option::is_none")]
+    /// Province
+    state: Option<String>,
+
+    #[serde(skip_serializing_if = "Option::is_none")]
+    /// Two-letter country code compliant with ISO-3166.
+    country_code: Option<String>,
+
+    #[serde(skip_serializing_if = "Option::is_none")]
+    /// Address description
+    name: Option<String>,
+
+    #[serde(skip_serializing_if = "Option::is_none")]
+    /// Recipient’s name
+    recipient_name: Option<String>,
+
+    #[serde(skip_serializing_if = "Option::is_none")]
+    /// Recipient’s e-mail address
+    recipient_email: Option<String>,
+
+    #[serde(skip_serializing_if = "Option::is_none")]
+    /// Recipient’s phone number
+    recipient_phone: Option<String>,
+}
+
+#[derive(Serialize, Deserialize, Debug, Default)]
+#[serde(rename_all = "camelCase")]
+pub struct BuyerShippingAddress {
+    #[serde(skip_serializing_if = "Option::is_none")]
+    /// stores the shipping address
+    delivery: Option<Delivery>,
+}
+
+impl BuyerShippingAddress {
+    pub fn new_with_delivery(delivery: Delivery) -> Self {
+        Self {
+            delivery: Some(delivery),
+        }
+    }
+}
+
 #[derive(Serialize, Deserialize, Debug, Default)]
 #[serde(rename_all = "camelCase")]
 pub struct Buyer {
     /// Required customer e-mail
-    email: String,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    email: Option<String>,
     /// Required customer phone number
-    phone: String,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    phone: Option<String>,
     /// Required customer first name
-    first_name: String,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    first_name: Option<String>,
     /// Required customer last name
-    last_name: String,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    last_name: Option<String>,
     /// Required customer language
-    language: String,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    language: Option<String>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    delivery: Option<BuyerShippingAddress>,
 }
 
 impl Buyer {
@@ -227,62 +296,68 @@ impl Buyer {
         Language: Into<String>,
     {
         Self {
-            email: email.into(),
-            phone: phone.into(),
-            first_name: first_name.into(),
-            last_name: last_name.into(),
-            language: lang.into(),
+            email: Some(email.into()),
+            phone: Some(phone.into()),
+            first_name: Some(first_name.into()),
+            last_name: Some(last_name.into()),
+            language: Some(lang.into()),
+            delivery: None,
         }
     }
 
     pub fn email(&self) -> &str {
-        &self.email
+        self.email.as_deref().unwrap_or_default()
     }
     pub fn with_email<S>(mut self, email: S) -> Self
     where
         S: Into<String>,
     {
-        self.email = email.into();
+        self.email = Some(email.into());
         self
     }
     pub fn phone(&self) -> &str {
-        &self.phone
+        self.phone.as_deref().unwrap_or_default()
     }
     pub fn with_phone<S>(mut self, phone: S) -> Self
     where
         S: Into<String>,
     {
-        self.phone = phone.into();
+        self.phone = Some(phone.into());
         self
     }
     pub fn first_name(&self) -> &str {
-        &self.first_name
+        self.first_name.as_deref().unwrap_or_default()
     }
     pub fn with_first_name<S>(mut self, first_name: S) -> Self
     where
         S: Into<String>,
     {
-        self.first_name = first_name.into();
+        self.first_name = Some(first_name.into());
         self
     }
     pub fn last_name(&self) -> &str {
-        &self.last_name
+        self.last_name.as_deref().unwrap_or_default()
     }
     pub fn with_last_name<S>(mut self, last_name: S) -> Self
     where
         S: Into<String>,
     {
-        self.last_name = last_name.into();
+        self.last_name = Some(last_name.into());
         self
     }
     pub fn language(&self) -> &str {
-        &self.language
+        self.language.as_deref().unwrap_or_default()
     }
     pub fn with_language<S>(mut self, language: S) -> Self
     where
         S: Into<String>,
     {
-        self.language = language.into();
+        self.language = Some(language.into());
+        self
+    }
+
+    pub fn with_delivery(mut self, delivery: Delivery) -> Self {
+        self.delivery = Some(BuyerShippingAddress::new_with_delivery(delivery));
         self
     }
 }
@@ -304,6 +379,14 @@ pub struct Product {
         deserialize_with = "deserialize::deserialize_u32"
     )]
     pub quantity: Quantity,
+    /// Product type, which can be virtual or material; (possible values true or
+    /// false).
+    #[serde(rename = "virtual", skip_serializing_if = "Option::is_none")]
+    pub virtual_product: Option<bool>,
+    /// Marketplace date from which the product (or offer) is available, ISO
+    /// format applies, e.g. "2019-03-27T10:57:59.000+01:00".
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub listing_date: Option<chrono::NaiveDateTime>,
 }
 
 impl Product {
@@ -312,13 +395,339 @@ impl Product {
             name: name.into(),
             unit_price,
             quantity,
+            virtual_product: None,
+            listing_date: None,
         }
     }
+
+    /// Product type, which can be virtual or material; (possible values true or
+    /// false).
+    pub fn into_virtual(mut self) -> Self {
+        self.virtual_product = Some(true);
+        self
+    }
+
+    /// Product type, which can be virtual or material; (possible values true or
+    /// false).
+    pub fn non_virtual(mut self) -> Self {
+        self.virtual_product = Some(false);
+        self
+    }
+
+    /// Marketplace date from which the product (or offer) is available, ISO
+    /// format applies, e.g. "2019-03-27T10:57:59.000+01:00".
+    pub fn with_listing_date(mut self, listing_date: chrono::NaiveDateTime) -> Self {
+        self.listing_date = Some(listing_date);
+        self
+    }
+
+    fn erase_listing_date(mut self) -> Self {
+        self.listing_date = None;
+        self
+    }
+}
+
+#[derive(Serialize, Deserialize, Debug)]
+#[serde(rename_all = "camelCase")]
+pub struct ShoppingCart {
+    /// Section containing data of shipping method.
+    #[serde(skip_serializing_if = "Option::is_none")]
+    shopping_method: Option<ShoppingMethod>,
+    /// Section containing data about ordered products.
+    /// > Note: product objects in the <shoppingCart.products> section do not
+    /// > have a listingDate field
+    #[serde(skip_serializing_if = "Option::is_none")]
+    products: Option<Vec<Product>>,
+    /// Submerchant identifier. This field should be consistent with field
+    /// extCustomerId in shoppingCarts section when order is placed in
+    /// marketplace.
+    ext_customer_id: String,
+}
+
+impl ShoppingCart {
+    pub fn new<ExtCustomerId, Products>(ext_customer_id: ExtCustomerId) -> Self
+    where
+        ExtCustomerId: Into<String>,
+    {
+        Self {
+            shopping_method: None,
+            ext_customer_id: ext_customer_id.into(),
+            products: None,
+        }
+    }
+
+    pub fn new_with_products<ExtCustomerId, Products>(
+        ext_customer_id: ExtCustomerId,
+        products: Products,
+    ) -> Self
+    where
+        ExtCustomerId: Into<String>,
+        Products: Iterator<Item = Product>,
+    {
+        Self {
+            shopping_method: None,
+            ext_customer_id: ext_customer_id.into(),
+            products: Some(products.map(Product::erase_listing_date).collect()),
+        }
+    }
+
+    pub fn with_products<Products>(mut self, products: Products) -> Self
+    where
+        Products: Iterator<Item = Product>,
+    {
+        self.products = Some(products.map(Product::erase_listing_date).collect());
+        self
+    }
+
+    /// Section containing data of shipping method.
+    pub fn shopping_method(&self) -> &Option<ShoppingMethod> {
+        &self.shopping_method
+    }
+
+    /// Section containing data about ordered products.
+    /// > Note: product objects in the <shoppingCart.products> section do not
+    /// > have a listingDate field
+    pub fn products(&self) -> &Option<Vec<Product>> {
+        &self.products
+    }
+
+    /// Submerchant identifier. This field should be consistent with field
+    /// extCustomerId in shoppingCarts section when order is placed in
+    /// marketplace.
+    pub fn ext_customer_id(&self) -> &String {
+        &self.ext_customer_id
+    }
+}
+
+/// Type of shipment
+#[derive(Serialize, Deserialize, Debug)]
+#[serde(rename_all = "SCREAMING_SNAKE_CASE")]
+pub enum ShoppingMethodType {
+    Courier,
+    CollectionPointPickup,
+    ParcelLocker,
+    StorePickup,
+}
+
+#[derive(Serialize, Deserialize, Debug)]
+#[serde(rename_all = "camelCase")]
+pub struct Address {
+    /// The full name of the pickup point, including its unique identifier, e.g.
+    /// „Parcel locker POZ29A”.
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub point_id: Option<String>,
+    /// Street name, possibly including house and flat number.
+    /// Recommended
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub street: Option<String>,
+    /// Street number
+    /// Recommended
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub street_no: Option<String>,
+    /// Flat number
+    /// Recommended
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub flat_no: Option<String>,
+    /// Postal code
+    /// Recommended
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub postal_code: Option<String>,
+    /// City
+    /// Recommended
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub city: Option<String>,
+    /// Two-letter country code compliant with ISO-3166
+    /// Recommended
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub country_code: Option<String>,
+}
+
+#[derive(Serialize, Deserialize, Debug)]
+#[serde(rename_all = "camelCase")]
+pub struct ShoppingMethod {
+    /// Shipping type
+    /// Recommended
+    #[serde(rename = "type", skip_serializing_if = "Option::is_none")]
+    pub shopping_type: Option<ShoppingMethodType>,
+    /// Shipping cost
+    /// Recommended
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub price: Option<String>,
+    /// Section containing data about shipping address.
+    /// Recommended
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub address: Option<Address>,
+}
+
+#[derive(Serialize, Deserialize, Debug)]
+#[serde(rename_all = "camelCase")]
+pub struct ApplicantAdditionalInfo {
+    /// Information whether there were previous, successfully completed orders
+    /// for applicant.
+    /// Recommended
+    #[serde(skip_serializing_if = "Option::is_none")]
+    has_successfully_finished_order_in_shop: Option<String>,
+}
+
+#[derive(Serialize, Deserialize, Debug)]
+#[serde(rename_all = "camelCase")]
+pub struct Applicant {
+    /// Applicant’s email address
+    /// Recommended
+    #[serde(skip_serializing_if = "Option::is_none")]
+    email: Option<String>,
+    /// Applicant’s phone number
+    /// Recommended
+    #[serde(skip_serializing_if = "Option::is_none")]
+    phone: Option<String>,
+    /// Applicant’s first name
+    /// Recommended
+    #[serde(skip_serializing_if = "Option::is_none")]
+    first_name: Option<String>,
+    /// Applicant’s last name
+    /// Recommended
+    #[serde(skip_serializing_if = "Option::is_none")]
+    last_name: Option<String>,
+    /// Language code, ISO-639-1 compliant. Denotes the language version of
+    /// PayU hosted payment page and of e-mail messages sent from PayU to the
+    /// payer (supported values are here).
+    /// Recommended
+    #[serde(skip_serializing_if = "Option::is_none")]
+    language: Option<String>,
+    /// National Identification Number
+    /// Recommended
+    #[serde(skip_serializing_if = "Option::is_none")]
+    nin: Option<String>,
+    /// Section containing data about applicant’s address.
+    /// Recommended
+    #[serde(skip_serializing_if = "Option::is_none")]
+    address: Option<Address>,
+    /// Additional information about person applying for credit.
+    /// Recommended
+    #[serde(skip_serializing_if = "Option::is_none")]
+    additional_info: Option<ApplicantAdditionalInfo>,
+}
+
+#[derive(Serialize, Deserialize, Debug, Default)]
+#[serde(rename_all = "camelCase")]
+pub struct Credit {
+    /// Section containing data of the ordered products
+    #[serde(skip_serializing_if = "Option::is_none")]
+    shopping_carts: Option<Vec<ShoppingCart>>,
+    /// Section containing data of person applying for a credit
+    #[serde(skip_serializing_if = "Option::is_none")]
+    applicant: Option<Applicant>,
+}
+
+impl Credit {
+    pub fn with_shopping_carts<ShoppingCarts>(mut self, shopping_carts: ShoppingCarts) -> Self
+    where
+        ShoppingCarts: Iterator<Item = ShoppingCart>,
+    {
+        self.shopping_carts = Some(shopping_carts.collect());
+        self
+    }
+}
+
+/// MultiUseCartToken
+pub mod muct {
+    use serde::{Deserialize, Serialize};
+
+    #[derive(Serialize, Deserialize, Debug)]
+    #[serde(rename_all = "SCREAMING_SNAKE_CASE")]
+    pub enum CardOnFile {
+        /// Payment initialized by the card owner who agreed to save card for
+        /// future use. You can expect full authentication (3D Secure
+        /// and/or CVV). If you want to use multi-use token (TOKC_)
+        /// later, you have to be confident, that first payment was
+        /// successful. Default value for single-use token (TOK_).
+        ///
+        /// In case of plain card data payments you should retrieve transaction
+        /// data to obtain first TransactionId. It should be passed in
+        /// payMethods.payMethod.card section for transactions marked as
+        /// STANDARD, STANDARD_CARDHOLDER and STANDARD_MERCHANT;
+        /// STANDARD_CARDHOLDER - payment with already saved card,
+        /// initialized by the card owner. This transaction has
+        /// multi-use token (TOKC_). Depending of payment parameters
+        /// (e.g. high transaction amount) strong authentication can be
+        /// expected (3D Secure and/or CVV). Default value for multi-use token
+        /// (TOKC_);
+        First,
+        /// Payment with already saved card, initialized by the shop without the
+        /// card owner participation. This transaction has multi-use token
+        /// (TOKC_). By the definition, this payment type does not
+        /// require strong authentication. You cannot use it if FIRST
+        /// card-on-file payment failed.
+        StandardMerchant,
+    }
+
+    #[derive(Serialize, Deserialize, Debug)]
+    #[serde(rename_all = "SCREAMING_SNAKE_CASE")]
+    pub enum Recurring {
+        /// Payment initialized by the card owner who agreed to save card for
+        /// future use in recurring plan. You can expect full authentication (3D
+        /// Secure and/or CVV). If you want to use multi-use token (TOKC_)
+        /// later, you have to be confident, that   first recurring
+        /// payment was successful.
+        First,
+        /// Subsequent recurring payment (user is not present). This transaction
+        /// has multi use token (TOKC_). You cannot use it if FIRST recurring
+        /// payment failed.
+        Standard,
+    }
+
+    #[derive(Serialize, Deserialize, Debug)]
+    #[serde(rename_all = "camelCase")]
+    pub struct MultiUseCartToken {
+        /// Information about party initializing order:
+        ///
+        /// * `FIRST` - payment initialized by the card owner who agreed to save
+        ///   card for future use. You can expect full authentication (3D Secure
+        ///   and/or CVV). If you want to use multi-use token (TOKC_) later, you
+        ///   have to be confident, that first payment was successful. Default
+        ///   value for single-use token (TOK_).
+        ///
+        ///   In case of plain card data payments you should retrieve
+        /// transaction   data to obtain first TransactionId. It should
+        /// be passed in   payMethods.payMethod.card section for
+        /// transactions marked as STANDARD,   STANDARD_CARDHOLDER and
+        /// STANDARD_MERCHANT; STANDARD_CARDHOLDER -   payment with
+        /// already saved card, initialized by the card owner. This
+        ///   transaction has multi-use token (TOKC_). Depending of payment
+        ///   parameters (e.g. high transaction amount) strong authentication
+        /// can be   expected (3D Secure and/or CVV). Default value for
+        /// multi-use token   (TOKC_);
+        /// * `STANDARD_MERCHANT` - payment with already saved card, initialized
+        ///   by the shop without the card owner participation. This transaction
+        ///   has multi-use token (TOKC_). By the definition, this payment type
+        ///   does not require strong authentication. You cannot use it if FIRST
+        ///   card-on-file payment failed.
+        ///
+        /// `cardOnFile` parameter cannot be used with recurring parameter.
+        pub card_on_file: CardOnFile,
+        /// Marks the order as recurring payment.
+        ///
+        /// * `FIRST` - payment initialized by the card owner who agreed to save
+        ///   card for future use in recurring plan. You can expect full
+        ///   authentication (3D Secure and/or CVV). If you want to use
+        ///   multi-use token (TOKC_) later, you have to be confident, that
+        ///   first recurring payment was successful.
+        /// * `STANDARD` - subsequent recurring payment (user is not present).
+        ///   This transaction has multi use token (TOKC_). You cannot use it if
+        ///   FIRST recurring payment failed.
+        ///
+        /// `recurring` parameter cannot be used with cardOnFile parameter.
+        pub recurring: Recurring,
+    }
 }
 
 #[derive(Serialize, Deserialize, Debug)]
 #[serde(rename_all = "camelCase")]
 pub struct OrderCreateRequest {
+    /// ID of an order used in merchant system. Order identifier assigned by the
+    /// merchant. It enables merchants to find a specific order in their system.
+    /// This value must be unique within a single POS.
+    ext_order_id: Option<String>,
     /// URL to which web hook will be send. It's important to return 200 to all
     /// notifications.
     ///
@@ -352,7 +761,22 @@ pub struct OrderCreateRequest {
     /// | 20| 72 hours |
     #[serde(skip_serializing_if = "Option::is_none")]
     notify_url: Option<String>,
-    /// Customer client IP address
+    /// Address for redirecting the customer after payment is commenced. If the
+    /// payment has not been authorized, error=501 parameter will be added.
+    /// Please note that no decision regarding payment status should be made
+    /// depending on the presence or lack of this parameter (to get payment
+    /// status, wait for notification or retrieve order details).
+    ///
+    /// IMPORTANT: the address must be compliant with the structure below:
+    /// <img src="https://developers.payu.com/images/continueUrlStructure_en.png" />
+    ///
+    /// Please keep in mind:
+    /// * accepted schemas are http and https,
+    /// * such elements as port, path, query and fragment are optional,
+    /// * query values must be encoded.
+    #[serde(skip_serializing_if = "Option::is_none")]
+    continue_url: Option<String>,
+    /// Payer’s IP address, e.g. 123.123.123.123. Note: 0.0.0.0 is not accepted.
     customer_ip: String,
     /// Secret pos ip. This is connected to PayU account
     #[serde(
@@ -377,29 +801,90 @@ pub struct OrderCreateRequest {
     products: Vec<Product>,
     #[serde(skip_serializing)]
     order_create_date: Option<String>,
+    /// Duration for the validity of an order (in seconds), during which time
+    /// payment must be made. Default value 86400.
+    #[serde(skip_serializing_if = "Option::is_none")]
+    validity_time: Option<u16>,
+    ///  Additional description of the order.
+    #[serde(skip_serializing_if = "Option::is_none")]
+    additional_description: Option<String>,
+    /// Text visible on the PayU payment page (max. 80 chars).
+    #[serde(skip_serializing_if = "Option::is_none")]
+    visible_description: Option<String>,
+    /// Payment recipient name followed by payment description (order ID, ticket
+    /// number etc) visible on card statement (max. 22 chars). The name should
+    /// be easy to recognize by the cardholder (e.g "shop.com 124343"). If field
+    /// is not provided, static name configured by PayU will be used.
+    #[serde(skip_serializing_if = "Option::is_none")]
+    statement_description: Option<String>,
+    #[serde(flatten, skip_serializing_if = "Option::is_none")]
+    muct: Option<muct::MultiUseCartToken>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    credit: Option<Credit>,
 }
 
 impl OrderCreateRequest {
-    pub fn new<CustomerIp, Currency>(
+    pub fn build<CustomerIp, Currency, Description>(
         buyer: Buyer,
         customer_ip: CustomerIp,
         currency: Currency,
-    ) -> Self
+        description: Description,
+    ) -> Result<Self>
     where
         CustomerIp: Into<String>,
         Currency: Into<String>,
+        Description: Into<String>,
     {
-        Self {
+        let customer_ip = customer_ip.into();
+        if &customer_ip == "0.0.0.0" {
+            return Err(Error::CustomerIp);
+        }
+        Ok(Self {
+            ext_order_id: None,
             notify_url: None,
-            customer_ip: customer_ip.into(),
+            continue_url: None,
+            customer_ip,
             merchant_pos_id: 0.into(),
-            description: String::from(""),
+            description: description.into(),
             currency_code: currency.into(),
             total_amount: 0,
             buyer: Some(buyer),
             products: Vec::new(),
             order_create_date: None,
-        }
+            validity_time: None,
+            additional_description: None,
+            visible_description: None,
+            statement_description: None,
+            muct: None,
+            credit: None,
+        })
+    }
+
+    /// ID of an order used in merchant system. Order identifier assigned by the
+    /// merchant. It enables merchants to find a specific order in their system.
+    /// This value must be unique within a single POS.
+    pub fn with_ext_order_id<S: Into<String>>(mut self, ext_order_id: S) -> Self {
+        self.ext_order_id = Some(ext_order_id.into());
+        self
+    }
+
+    /// Duration for the validity of an order (in seconds), during which time
+    /// payment must be made. Default value 86400.
+    pub fn with_validity_time(mut self, validity_time: u16) -> Self {
+        self.validity_time = Some(validity_time);
+        self
+    }
+
+    pub fn with_multi_use_token(
+        mut self,
+        recurring: muct::Recurring,
+        card_on_file: muct::CardOnFile,
+    ) -> Self {
+        self.muct = Some(muct::MultiUseCartToken {
+            recurring,
+            card_on_file,
+        });
+        self
     }
 
     pub fn with_products<Products>(mut self, products: Products) -> Self
@@ -423,6 +908,9 @@ impl OrderCreateRequest {
         self
     }
 
+    /// Description of the an order.
+    ///
+    /// > This method will override initial description!
     pub fn with_description<Description>(mut self, desc: Description) -> Self
     where
         Description: Into<String>,
@@ -431,6 +919,38 @@ impl OrderCreateRequest {
         self
     }
 
+    /// Additional description of the order.
+    pub fn with_additional_description<S: Into<String>>(
+        mut self,
+        additional_description: S,
+    ) -> Self {
+        self.additional_description = Some(additional_description.into());
+        self
+    }
+
+    /// Text visible on the PayU payment page (max. 80 chars).
+    pub fn with_visible_description(mut self, visible_description: &str) -> Self {
+        let visible_description = if visible_description.len() > 60 {
+            &visible_description[..60]
+        } else {
+            visible_description
+        };
+        self.visible_description = Some(String::from(visible_description));
+        self
+    }
+
+    /// Payment recipient name followed by payment description (order ID, ticket
+    /// number etc) visible on card statement (max. 22 chars). The name should
+    /// be easy to recognize by the cardholder (e.g "shop.com 124343"). If field
+    /// is not provided, static name configured by PayU will be used.
+    pub fn with_statement_description<Description>(mut self, desc: Description) -> Self
+    where
+        Description: Into<String>,
+    {
+        self.statement_description = Some(String::from(desc.into().trim()));
+        self
+    }
+
     /// Add url to which PayU will be able to send http request with payment
     /// status updates
     ///
@@ -445,6 +965,28 @@ impl OrderCreateRequest {
         self
     }
 
+    /// Address for redirecting the customer after payment is commenced. If the
+    /// payment has not been authorized, error=501 parameter will be added.
+    /// Please note that no decision regarding payment status should be made
+    /// depending on the presence or lack of this parameter (to get payment
+    /// status, wait for notification or retrieve order details).
+    pub fn with_continue_url<ContinueUrl>(mut self, continue_url: ContinueUrl) -> Self
+    where
+        ContinueUrl: Into<String>,
+    {
+        self.continue_url = Some(continue_url.into());
+        self
+    }
+
+    /// Section containing credit data. This information is not required, but it
+    /// is strongly recommended to include it. Otherwise the buyer will be
+    /// prompted to provide missing data on provider page when payment by
+    /// Installments or Pay later.
+    pub fn with_credit(mut self, credit: Credit) -> Self {
+        self.credit = Some(credit);
+        self
+    }
+
     /// URL to which web hook will be send. It's important to return 200 to all
     /// notifications.
     ///
@@ -485,25 +1027,30 @@ impl OrderCreateRequest {
         &self.customer_ip
     }
 
-    ///
     pub fn merchant_pos_id(&self) -> MerchantPosId {
         self.merchant_pos_id
     }
+
     pub fn description(&self) -> &String {
         &self.description
     }
+
     pub fn currency_code(&self) -> &String {
         &self.currency_code
     }
+
     pub fn total_amount(&self) -> &Price {
         &self.total_amount
     }
+
     pub fn buyer(&self) -> &Option<Buyer> {
         &self.buyer
     }
+
     pub fn products(&self) -> &[Product] {
         &self.products
     }
+
     pub fn order_create_date(&self) -> &Option<String> {
         &self.order_create_date
     }
